# Standard Library
from typing import Sequence, Union

# Third Party Library
from attrs import define

# First Party Library
from peru_dnie.i18n import t

ByteLike = Union[Sequence[int], bytes, bytearray]


class APDUError(Exception):
    """Error due from APDU transactions"""


@define
class APDUCommand:
    ins: int
    p1: int
    p2: int
    lc: Union[int, None] = None
    data: Union[bytes, None] = None
    le: Union[int, None] = None
    cla: int = 0x00

    def __attrs_post_init__(self):
        if self.data is None and self.lc is not None:
            raise ValueError(t["errors"]["lc_must_none"])

        elif self.data is not None and self.lc != len(self.data):
            raise ValueError(t["errors"]["lc_must_length"])

    def serialize(self) -> bytes:
        command = bytes([self.cla, self.ins, self.p1, self.p2])

        if self.lc is not None and self.data is not None:
            command += self.lc.to_bytes(length=1, byteorder="big") + self.data

        if self.le is not None:
            command += self.le.to_bytes(length=1, byteorder="big")

        return command


@define
class APDUResponse:
    sw1: int
    sw2: int
    data: Union[bytes, None] = None

    @property
    def ok(self):
        return (self.sw1, self.sw2) == (0x90, 0x00)



# Standard Library
import locale
import os


def get_current_language():
    default_locale = locale.getdefaultlocale()
    if len(default_locale) > 0 and default_locale[0] is not None:
        current_lang = default_locale[0].split("_")[0]
        if current_lang not in ["en", "es"]:
            current_lang = "en"
    else:
        current_lang = "en"

    force_lang = os.getenv("PERUDNIE_LANG", None)
    if force_lang is not None:
        current_lang = force_lang

    return current_lang


LANG = {
    "en": {
        "cli": {
            "sign": {
                "sign_help": "Sign with the DNIe",
                "input_file_help": "File to sign",
                "output_file_help": "Output signature file",
                "hash_algorithm_help": "Hash algorithm for the signature",
            },
            "extract": {
                "extract_help": "Extract certificates from the DNIe",
                "certificate_type_help": "Type of certificate to extract",
                "output_file_help": "Output public certificate file",
            },
            "program_description": "Utilities for the Peruvian DNIe Smart Card cryptographic functions",
            "available_tasks": "Available DNIe tasks",
        },
        "init": {
            "choose_reader": "Choose reader",
            "readers": "[bold underline]Readers:",
            "waiting_dnie": "Waiting for DNIe...",
            "found_dnie": "[green]Found DNIe V2",
        },
        "certificates": {
            "reading_cert": "Reading signature certificate...",
            "success": "[green]Certificate successfully loaded",
            "failed": "[red]Could not read certificate",
            "wrote_cert": "[green]Wrote certificate to '{}'",
        },
        "general": {
            "enter_pin": "Please enter your PIN",
        },
        "errors": {
            "lc_must_none": "'lc' must be None if 'data' is None",
            "lc_must_length": "'lc' must be the length of 'data'",
            "dnie_not_init": "DNIe card is not initialized",
            "dnie_not_found": "Could not find DNIe",
            "could_not_select_pki": "Could not select PKI app: '{}'",
            "could_not_select_cert": "Could not select signature certificate file: '{}'",
            "could_not_read_cert": "Could not read certificate: '{}'",
            "wrong_while_reading": "Something went wrong while reading the certificate: '{}'",
            "certificate_not_supported": "Certificate type extraction not supported",
            "failed_pin": "Failed to verify PIN: '{}'",
            "could_not_set_env": "Could not set security environment: '{}'",
            "could_not_sign": "Could not sign payload: '{}'",
        },
    },
    "es": {
        "cli": {
            "sign": {
                "sign_help": "Firmar con el DNIe",
                "input_file_help": "Archivo a firmar",
                "output_file_help": "Archivo de firma resultante",
                "hash_algorithm_help": "Algoritmo de hash para la firma",
            },
            "extract": {
                "extract_help": "Extraer certificados del DNIe",
                "certificate_type_help": "Tipo de certificado a extraer",
                "output_file_help": "Archivo de certificado público resultante",
            },
            "program_description": "Utilidades para las funciones criptográficas de la Tarjeta Inteligente DNIe peruana",
            "available_tasks": "Tareas DNIe disponibles",
        },
        "init": {
            "choose_reader": "Elegir lector",
            "readers": "[bold underline]Lectores:",
            "waiting_dnie": "Esperando el DNIe...",
            "found_dnie": "[green]DNIe V2 encontrado",
        },
        "certificates": {
            "reading_cert": "Leyendo certificado de firma...",
            "success": "[green]Certificado cargado con éxito",
            "failed": "[red]No se pudo leer el certificado",
            "wrote_cert": "[green]Certificado escrito en '{}'",
        },
        "general": {
            "enter_pin": "Por favor, introduce tu PIN",
        },
        "errors": {
            "lc_must_none": "'lc' debe ser None si 'data' es None",
            "lc_must_length": "'lc' debe ser la longitud de 'data'",
            "dnie_not_init": "La tarjeta DNIe no está inicializada",
            "dnie_not_found": "No se pudo encontrar el DNIe",
            "could_not_select_pki": "No se pudo seleccionar la aplicación PKI: '{:!r}'",
            "could_not_select_cert": "No se pudo seleccionar el archivo de certificado de firma: '{:!r}'",
            "could_not_read_cert": "No se pudo leer el certificado: '{:!r}'",
            "wrong_while_reading": "Algo salió mal al leer el certificado: '{:!r}'",
            "certificate_not_supported": "Extracción de tipo de certificado no soportada",
            "failed_pin": "Fallo al verificar el PIN: '{:!r}'",
            "could_not_set_env": "No se pudo configurar el entorno de seguridad: '{:!r}'",
            "could_not_sign": "No se pudo firmar el payload: '{:!r}'",
        },
    },
}

current_lang = get_current_language()
t = LANG[current_lang]



# Standard Library
import hashlib
from typing import Literal

# Third Party Library
from attrs import define

# First Party Library
from peru_dnie.constants import DER_HASH_ALGORITHM_ENCODINGS

HashTypes = Literal["sha224", "sha256", "sha384", "sha512"]


@define
class HashFunction:
    name: HashTypes

    def der_encoding(self) -> bytes:
        return DER_HASH_ALGORITHM_ENCODINGS[self.name]

    def __call__(self, input_bytes: bytes) -> bytes:
        if self.name == "sha224":
            return hash_sha224(input_bytes)
        elif self.name == "sha256":
            return hash_sha256(input_bytes)
        elif self.name == "sha384":
            return hash_sha384(input_bytes)
        elif self.name == "sha512":
            return hash_sha384(input_bytes)
        else:
            raise TypeError("Hash function not supported")


def hash_sha224(input_bytes: bytes) -> bytes:
    """Hash the input file for DNIe signature"""
    hash = hashlib.sha224(input_bytes).digest()

    return hash


def hash_sha256(input_bytes: bytes) -> bytes:
    """Hash the input file for DNIe signature"""
    hash = hashlib.sha256(input_bytes).digest()

    return hash


def hash_sha384(input_bytes: bytes) -> bytes:
    """Hash the input file for DNIe signature"""
    hash = hashlib.sha384(input_bytes).digest()

    return hash


def hash_sha512(input_bytes: bytes) -> bytes:
    """Hash the input file for DNIe signature"""
    hash = hashlib.sha512(input_bytes).digest()

    return hash



# Third Party Library
import pytest

# First Party Library
from peru_dnie.context import FakeContext
from peru_dnie.hashes import HashFunction


@pytest.fixture(scope="session")
def ctx():
    yield FakeContext(hash_func=HashFunction(name="sha256"))



# Standard Library
from typing import Final

# Third Party Library
from attrs import define
from rich.console import Console


@define
class CliConfig:
    console: Final = Console(color_system="truecolor")
    DEBUG: Final = False


CLI_CONFIG: Final = CliConfig()



# Standard Library
from typing import List

# Third Party Library
from attrs import define
from smartcard.CardRequest import CardRequest
from smartcard.CardType import CardType
from smartcard.pcsc.PCSCReader import PCSCCardConnection, PCSCReader
from smartcard.System import readers

# First Party Library
from peru_dnie.apdu import APDUCommand, APDUResponse
from peru_dnie.card import SmartCard
from peru_dnie.constants import PERU_DNIE_V2_ATR, PERU_DNIE_V2_ATR_NFC
from peru_dnie.exceptions import CardError
from peru_dnie.i18n import t


class DNIv2CardType(CardType):

    def __init__(self):
        super().__init__()

    def matches(self, atr, reader=None) -> bool:  # pyright: ignore[reportIncompatibleMethodOverride]
        if atr == PERU_DNIE_V2_ATR:
            return True
        elif atr == PERU_DNIE_V2_ATR_NFC:
            return True

        return False



@define
class PyscardSmartCard(SmartCard):
    def transmit(self, command: APDUCommand) -> APDUResponse:
        serial_bytes = command.serialize()
        data, sw1, sw2 = self.connection.transmit(list(serial_bytes))
        return APDUResponse(sw1=sw1, sw2=sw2, data=bytes(data))


def get_readers() -> List[PCSCReader]:
    """Get Smart Card readers"""
    current_readers = readers()
    return current_readers


def get_dnie_connection() -> PCSCCardConnection:
    """Get DNIe card connection.

    Wait until the DNIe is connected to the reader and return a connection.
    """
    card_request = CardRequest(timeout=120, cardType=DNIv2CardType())
    card_service = card_request.waitforcard()

    if card_service is not None:
        return card_service.connection
    else:
        raise CardError(t["errors"]["dnie_not_found"])



# Standard Library
from argparse import ArgumentParser
from pathlib import Path
from typing import get_args

# First Party Library
from peru_dnie.card_init import initialize_smart_card
from peru_dnie.commands.certificate import extract_certificate_to_file
from peru_dnie.commands.signature import sign_file
from peru_dnie.context import Context
from peru_dnie.hashes import HashFunction, HashTypes
from peru_dnie.i18n import t


def register_sign_parser(subparsers):
    sign_parser = subparsers.add_parser(
        "sign",
        help=t["cli"]["sign"]["sign_help"],
    )
    sign_parser.add_argument(
        "input_file",
        type=Path,
        help=t["cli"]["sign"]["input_file_help"],
    )
    sign_parser.add_argument(
        "output_file",
        type=Path,
        help=t["cli"]["sign"]["output_file_help"],
    )
    sign_parser.add_argument(
        "--hash-algorithm",
        default="sha256",
        choices=get_args(HashTypes),
        help=t["cli"]["sign"]["hash_algorithm_help"],
    )


def register_extract_certificate_parser(subparsers):
    certificate_parser = subparsers.add_parser(
        "extract",
        help=t["cli"]["extract"]["extract_help"],
    )
    certificate_parser.add_argument(
        "certificate_type",
        choices=["signature", "encryption"],
        type=str,
        help=t["cli"]["extract"]["certificate_type_help"],
    )
    certificate_parser.add_argument(
        "output_file",
        type=Path,
        help=t["cli"]["extract"]["output_file_help"],
    )


def main():
    parser = ArgumentParser(
        prog="dniectl",
        description=t["cli"]["program_description"],
    )

    subparsers = parser.add_subparsers(
        dest="command",
        help=t["cli"]["available_tasks"],
    )

    register_sign_parser(subparsers)
    register_extract_certificate_parser(subparsers)

    args = parser.parse_args()

    if args.command == "sign":
        ctx = Context(hash_func=HashFunction(name=args.hash_algorithm))
        initialize_smart_card(ctx)

        sign_file(
            ctx,
            input_file=args.input_file,
            output_file=args.output_file,
        )

    elif args.command == "extract":
        ctx = Context()
        initialize_smart_card(ctx)

        extract_certificate_to_file(
            ctx,
            output_file=args.output_file,
            certificate_type=args.certificate_type,
        )

    else:
        parser.print_help()



# Third Party Library
import pytest

# First Party Library
from peru_dnie.constants import DER_HASH_ALGORITHM_ENCODINGS
from peru_dnie.hashes import (
    HashFunction,
    hash_sha224,
    hash_sha256,
    hash_sha384,
    hash_sha512,
)


class Test_HashFunction:
    @pytest.mark.pointer(target=HashFunction.der_encoding)
    def test_der_encoding(self):
        hash_func = HashFunction(name="sha256")

        assert hash_func.der_encoding() == DER_HASH_ALGORITHM_ENCODINGS["sha256"]

    @pytest.mark.pointer(target=HashFunction.__call__)
    def test___call__(self):
        hash_func = HashFunction(name="sha256")
        some_data = b"some data here"
        hash = hash_func(some_data)

        assert hash == bytes.fromhex(
            "679aa02ff1852e40618b0701f430cad8bfcaa0811579edb17a258c2b322b9826"
        )


@pytest.mark.pointer(target=hash_sha224)
def test_hash_sha224():
    some_data = b"some data here"
    hash = hash_sha224(some_data)

    assert (
        hash
        == b"\x84l\x7fc\xc4\x9d\xb3\xa5}?N\xec\x0e2A\xeb?\xa3\xf9\xd70\xde-A\x98\xd0\xe8\x01"
    )


@pytest.mark.pointer(target=hash_sha256)
def test_hash_sha256():
    some_data = b"some data here"
    hash = hash_sha256(some_data)

    assert hash == bytes.fromhex(
        "679aa02ff1852e40618b0701f430cad8bfcaa0811579edb17a258c2b322b9826"
    )


@pytest.mark.pointer(target=hash_sha384)
def test_hash_sha384():
    some_data = b"some data here"
    hash = hash_sha384(some_data)

    assert hash == bytes.fromhex(
        "82f2d988f9e7acaf5def7a297b159247998e69a7e720bb7a4a1fb77d080731530d97cec50c003e929d72cd1c76baa52c"
    )


@pytest.mark.pointer(target=hash_sha512)
def test_hash_sha512():
    some_data = b"some data here"
    hash = hash_sha512(some_data)

    assert hash == bytes.fromhex(
        "c0b2a5f4e22cf4f23233ac51a47f522aa92ef49647be72eaa8e287fe663707678173c8038c2b3e3a1d870f8ae21b807a9d4917cb5247de10105984f441baf518"
    )



# Standard Library
from unittest.mock import MagicMock

# Third Party Library
import pytest

# First Party Library
from peru_dnie.commands import general
from peru_dnie.commands.signature import (
    PaddingSchemes,
    build_signature_payload,
    sign_bytes,
)
from peru_dnie.hashes import HashFunction


@pytest.mark.pointer(target=build_signature_payload)
def test_build_signature_payload():
    input_bytes = b"some information to sign"
    hash_sha256 = HashFunction(name="sha256")

    digest_info = build_signature_payload(
        input_bytes, hash_sha256, PaddingSchemes.PKCS1_15
    )

    assert digest_info == (
        hash_sha256.der_encoding()
        + bytes.fromhex(
            "0a409cc96995251f8ed39b45d1eabd159355a03f2f6784fe945d1c824804a8f3"
        )
    )


@pytest.mark.pointer(target=sign_bytes)
def test_sign_bytes(ctx):
    input_bytes = b"some information to sign"

    class FakePrompt:
        ask = MagicMock(return_value="1234")

    general.Prompt = FakePrompt

    signature = sign_bytes(ctx, input_bytes)

    assert signature == b"\xff" * 10



# Third Party Library
from rich.prompt import IntPrompt
from rich.status import Status

# First Party Library
from peru_dnie.context import Context
from peru_dnie.i18n import t
from peru_dnie.pyscard import PyscardSmartCard, get_dnie_connection, get_readers


def select_reader(ctx: Context):
    readers = get_readers()

    idx = IntPrompt.ask(t["init"]["choose_reader"], console=ctx.cli.console) - 1

    ctx.cli.console.print(t["init"]["readers"])
    for idx, reader in enumerate(readers):
        ctx.cli.console.print(f"[blue]{idx + 1}.[/] {reader}")


def initialize_smart_card(ctx: Context):
    spinner = Status(t["init"]["waiting_dnie"], console=ctx.cli.console)
    spinner.start()

    ctx.card = PyscardSmartCard(connection=get_dnie_connection())
    ctx.card.connection.connect()

    spinner.stop()
    ctx.cli.console.print(t["init"]["found_dnie"])



# Standard Library
from typing import Union

# Third Party Library
from attrs import define

# First Party Library
from peru_dnie.apdu import APDUCommand, APDUResponse
from peru_dnie.card import SmartCard
from peru_dnie.cli_config import CLI_CONFIG, CliConfig
from peru_dnie.hashes import HashFunction
from peru_dnie.i18n import t


@define
class Context:
    hash_func: Union[HashFunction, None] = None
    card: Union[SmartCard, None] = None
    cli: CliConfig = CLI_CONFIG

    def transmit(self, command: APDUCommand) -> APDUResponse:
        if self.card is None:
            raise RuntimeError(t["errors"]["dnie_not_init"])
        return self.card.transmit(command)


@define
class FakeContext(Context):
    def transmit(self, command: APDUCommand):
        if command.serialize()[:4] == bytes([0x00, 0x2A, 0x9E, 0x9A]):
            data = b"\xff" * 10
        else:
            data = None
        return APDUResponse(sw1=0x90, sw2=0x00, data=data)



# Standard Library
from enum import Enum
from typing import Final

# Third Party Library
from rich.prompt import Prompt

# First Party Library
from peru_dnie.apdu import APDUCommand
from peru_dnie.context import Context
from peru_dnie.i18n import t


class PinType(Enum):
    SIGNATURE = 0x81
    ENCRYPTION = 0x83


SELECT_PKI_APP: Final = APDUCommand(
    cla=0x00,
    ins=0xA4,
    p1=0x04,
    p2=0x00,
    lc=0x0E,
    data=bytes(
        [
            0xE8,
            0x28,
            0xBD,
            0x08,
            0x0F,
            0xD2,
            0x50,
            0x47,
            0x65,
            0x6E,
            0x65,
            0x72,
            0x69,
            0x63,
        ]
    ),
)


def verify_pin(ctx: Context, *, pin_type: PinType) -> bool:
    """Verify the PIN before a DNIe cryptographic operation"""
    pin = Prompt.ask(
        t["general"]["enter_pin"],
        password=True,
        console=ctx.cli.console,
    )

    encoded_pin = pin.encode("ascii")

    verify_command = APDUCommand(
        cla=0x00,
        ins=0x20,
        p1=0x00,
        p2=pin_type.value,
        lc=len(encoded_pin),
        data=encoded_pin,
    )

    r = ctx.transmit(verify_command)

    if not r.ok:
        raise RuntimeError(t["errors"]["failed_pin"].format(repr(r)))

    return True



# Standard Library
from abc import ABC, abstractmethod

# Third Party Library
from attrs import define
from smartcard.pcsc.PCSCReader import PCSCCardConnection

# First Party Library
from peru_dnie.apdu import APDUCommand, APDUResponse


@define
class SmartCard(ABC):
    connection: PCSCCardConnection

    @abstractmethod
    def transmit(self, command: APDUCommand) -> APDUResponse:
        pass



# Standard Library
from pathlib import Path

# Third Party Library
from rich.status import Status

# First Party Library
from peru_dnie.apdu import APDUCommand, APDUError
from peru_dnie.context import Context
from peru_dnie.i18n import t

# Local Modules
from .general import SELECT_PKI_APP


def extract_signature_certificate(ctx: Context) -> bytes:
    """Get signature x509 certificate from DNIe"""

    # Open PKI app
    r = ctx.transmit(SELECT_PKI_APP)

    if ctx.cli.DEBUG:
        print("Select PKI: '{r:!r}'")

    if not r.ok:
        raise APDUError(t["errors"]["could_not_select_pki"].format(repr(r)))

    # Select signature certificate
    select_signature_certificate = APDUCommand(
        cla=0x00,
        ins=0xA4,
        p1=0x02,
        p2=0x04,
        lc=0x02,
        data=bytes([0x00, 0x1D]),
    )
    r = ctx.transmit(select_signature_certificate)

    if ctx.cli.DEBUG:
        print("Select signature certificate: '{r:!r}'")

    if not r.ok:
        raise APDUError(t["errors"]["could_not_select_cert"].format(repr(r)))

    read_cert_apdu_command = APDUCommand(
        cla=0x00,
        ins=0xB1,
        p1=0x00,
        p2=0x00,
        lc=0x04,
        data=bytes([0x54, 0x02, 0x00, 0x00]),
        le=0xFF,
    )

    if read_cert_apdu_command.data is None:
        raise TypeError(
            f"Read certificate data APDU must have a data field '{read_cert_apdu_command:!r}'"
        )

    spinner = Status(t["certificates"]["reading_cert"], console=ctx.cli.console)
    spinner.start()

    output_certificate = b""
    success = False
    while True:
        r = ctx.transmit(read_cert_apdu_command)

        if r.data is None:
            raise APDUError(t["errors"]["could_not_read_cert"].format(repr(r)))

        # First two bytes are the tag. Third byte is length (should be 0xe4).
        # See TLV frame.
        output_certificate += r.data[3:]

        if ctx.cli.DEBUG:
            print("-------------------")
            print(f"Response '{r:!r}'")
            print("  ", "Data", r.data)
            print("  ", "Offset:", [hex(j) for j in read_cert_apdu_command.data])
            print("-------------------\n")

        # Break if Status Word is found
        if (r.sw1, r.sw2) == (0x62, 0x82):
            success = True
            break

        if r.data[0] != 0x53 or not r.ok:
            raise APDUError(t["errors"]["wrong_while_reading"].format(repr(r)))

        # Update reading command with new offset
        offset = int.from_bytes(read_cert_apdu_command.data[2:]) + 0xE4
        offset = offset.to_bytes(length=2, byteorder="big")
        read_cert_apdu_command.data = read_cert_apdu_command.data[:2] + offset

    if success:
        spinner.stop()
        ctx.cli.console.print(t["certificates"]["success"])
    else:
        ctx.cli.console.print(t["certificates"]["failed"])
        raise SystemExit()

    return output_certificate


def extract_certificate_to_file(
    ctx: Context,
    *,
    output_file: Path,
    certificate_type: str,
):
    if certificate_type == "signature":
        certificate = extract_signature_certificate(ctx)
    else:
        raise TypeError(t["errors"]["certificate_not_supported"])

    output_file.write_bytes(certificate)
    ctx.cli.console.print(t["certificates"]["wrote_cert"].format(output_file.name))



# Standard Library
from enum import Enum
from pathlib import Path

# First Party Library
from peru_dnie.apdu import APDUCommand, APDUError
from peru_dnie.context import Context
from peru_dnie.hashes import HashFunction
from peru_dnie.i18n import t

# Local Modules
from .general import SELECT_PKI_APP, PinType, verify_pin


class PaddingSchemes(str, Enum):
    PKCS1_15 = "pkcs1_15"


def build_signature_payload(
    input_bytes: bytes,
    hash_func: HashFunction,
    padding_scheme: PaddingSchemes,
) -> bytes:
    """Prepare a PKCS padded sha256sum hash

    Source <https://datatracker.ietf.org/doc/html/rfc3447#section-9.2>

    Peruvian DNIe v2 is configured for padding the digest info payload. There is no need
    for filling with 0x00 0x01 0xff.. 0xff 0x00 digest_info.
    """

    hashed_bytes = hash_func(input_bytes)
    if padding_scheme == PaddingSchemes.PKCS1_15:
        digest_info = hash_func.der_encoding() + hashed_bytes
    else:
        raise TypeError("Padding scheme not supported")

    return digest_info


def sign_bytes(
    ctx: Context,
    input_bytes: bytes,
) -> bytes:
    """Sign a string of bytes with the DNIe card."""

    if ctx.hash_func is None:
        raise ValueError("A hash function is needed for a signature.")

    pkcs1_15_padded_hash = build_signature_payload(
        input_bytes,
        ctx.hash_func,
        PaddingSchemes.PKCS1_15,
    )

    r = ctx.transmit(SELECT_PKI_APP)

    if not r.ok:
        raise APDUError(t["errors"]["could_not_select_pki"].format(repr(r)))

    verify_pin(ctx, pin_type=PinType.SIGNATURE)

    # Set security enviroment for RSA signature, off-card hashing, and in card
    # padding. With these settings the card expects a hash with its corresponding
    # digest_info hash identifier. The padding bytes are added by the card.
    set_security_environment = APDUCommand(
        cla=0x00,
        ins=0x22,
        p1=0x41,
        p2=0xB6,
        lc=0x06,
        data=bytes(
            [
                0x80,
                0x01,
                0x8A,
                0x84,
                0x01,
                0x81,
            ]
        ),
    )

    r = ctx.transmit(set_security_environment)

    if not r.ok:
        raise APDUError(t["errors"]["could_not_set_env"].format(repr(r)))

    signature_command = APDUCommand(
        cla=0x00,
        ins=0x2A,
        p1=0x9E,
        p2=0x9A,
        lc=len(pkcs1_15_padded_hash),
        data=pkcs1_15_padded_hash,
    )

    r = ctx.transmit(signature_command)

    if not r.ok or r.data is None:
        raise APDUError(t["errors"]["could_not_sign"].format(repr(r)))

    return r.data


def sign_file(
    ctx: Context,
    *,
    input_file: Path,
    output_file: Path,
) -> None:
    """Sign a file with the DNIe"""

    input_bytes = input_file.read_bytes()

    signature = sign_bytes(ctx, input_bytes)

    output_file.write_bytes(signature)
